#include <Adafruit_GFX.h>
#include <Adafruit_TFTLCD.h>
#include <TouchScreen.h>

#if defined(__SAM3X8E__)
    #undef __FlashStringHelper::F(string_literal)
    #define F(string_literal) string_literal
#endif

// When using the BREAKOUT BOARD only, use these 8 data lines to the LCD:
// For the Arduino Uno, Duemilanove, Diecimila, etc.:
//   D0 connects to digital pin 8  (Notice these are
//   D1 connects to digital pin 9   NOT in order!)
//   D2 connects to digital pin 2
//   D3 connects to digital pin 3
//   D4 connects to digital pin 4
//   D5 connects to digital pin 5
//   D6 connects to digital pin 6
//   D7 connects to digital pin 7

// For the Arduino Mega, use digital pins 22 through 29
// (on the 2-row header at the end of the board).
//   D0 connects to digital pin 22
//   D1 connects to digital pin 23
//   D2 connects to digital pin 24
//   D3 connects to digital pin 25
//   D4 connects to digital pin 26
//   D5 connects to digital pin 27
//   D6 connects to digital pin 28
//   D7 connects to digital pin 29

// For the Arduino Due, use digital pins 33 through 40
// (on the 2-row header at the end of the board).
//   D0 connects to digital pin 33
//   D1 connects to digital pin 34
//   D2 connects to digital pin 35
//   D3 connects to digital pin 36
//   D4 connects to digital pin 37
//   D5 connects to digital pin 38
//   D6 connects to digital pin 39
//   D7 connects to digital pin 40
/*
#define YP 9  // must be an analog pin, use "An" notation!
#define XM 8  // must be an analog pin, use "An" notation!
#define YM A2   // can be a digital pin
#define XP A3   // can be a digital pin
*/

#define YP A3  // must be an analog pin, use "An" notation!
#define XM A2  // must be an analog pin, use "An" notation!
#define YM 9   // can be a digital pin
#define XP 8   // can be a digital pin

#define TS_MINX 0
#define TS_MINY 0
#define TS_MAXX 800
#define TS_MAXY 820

// For better pressure precision, we need to know the resistance
// between X+ and X- Use any multimeter to read it
// For the one we're using, its 300 ohms across the X plate
TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300);

#define LCD_CS A3
#define LCD_CD A2
#define LCD_WR A1
#define LCD_RD A0
// optional
#define LCD_RESET A4

// Assign human-readable names to some common 16-bit color values:
#define	BLACK   0x0000
#define	BLUE    0x001F
#define	RED     0xF800
#define	GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

Adafruit_TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET);

#define PENRADIUS 3

int g_check[ 5 ][ 4 ] = {
  { 3, 3, TS_MAXX, TS_MINY },
  { 3, 317, TS_MAXX, TS_MAXY },
  { 237, 317, TS_MINX, TS_MAXY },
  { 237, 3, TS_MINX, TS_MINY },
  { 120, 160, TS_MAXX / 2, TS_MAXY / 2 }
};

int g_checkIndex( 0 );

int g_colors[ 5 ] = {
  BLUE, RED, GREEN, CYAN, MAGENTA
};

TSPoint g_cali;

void setup(void) {
  Serial.begin( 9600 );
  
  tft.reset();
  
  uint16_t identifier( tft.readID() );
  if(identifier == 0x9325) {
    Serial.println(F("Found ILI9325 LCD driver"));
  } else if(identifier == 0x9328) {
    Serial.println(F("Found ILI9328 LCD driver"));
  } else if(identifier == 0x4535) {
    Serial.println(F("Found LGDP4535 LCD driver"));
  }else if(identifier == 0x7575) {
    Serial.println(F("Found HX8347G LCD driver"));
  } else if(identifier == 0x9341) {
    Serial.println(F("Found ILI9341 LCD driver"));
  } else if(identifier == 0x8357) {
    Serial.println(F("Found HX8357D LCD driver"));
  } else if(identifier==0x0101) {     
      identifier=0x9341;
       Serial.println(F("Found 0x9341 LCD driver"));
  } else {
    Serial.print(F("Unknown LCD driver chip: "));
    Serial.println(identifier, HEX);
    Serial.println(F("If using the Adafruit 2.8\" TFT Arduino shield, the line:"));
    Serial.println(F("  #define USE_ADAFRUIT_SHIELD_PINOUT"));
    Serial.println(F("should appear in the library header (Adafruit_TFT.h)."));
    Serial.println(F("If using the breakout board, it should NOT be #defined!"));
    Serial.println(F("Also if using the breakout, double-check that all wiring"));
    Serial.println(F("matches the tutorial."));
    identifier=0x9341;
  }

  tft.begin( identifier );
  tft.setRotation( 2 );
  tft.fillScreen( BLACK );
 
  pinMode( 13, OUTPUT );
  tft.fillCircle( g_check[ g_checkIndex ][0], g_check[ g_checkIndex ][1], PENRADIUS, g_colors[ g_checkIndex ] );

  g_cali.x = 0;
  g_cali.y = 0;
}

#define MINPRESSURE 10
#define MAXPRESSURE 1000

void loop()
{
  digitalWrite( 13, HIGH );
  TSPoint p( ts.getPoint() );
  digitalWrite( 13, LOW) ;

  // if sharing pins, you'll need to fix the directions of the touchscreen pins
  //pinMode(XP, OUTPUT);
  pinMode( XM, OUTPUT );
  pinMode( YP, OUTPUT );
  //pinMode(YM, OUTPUT);

  // we have some minimum pressure we consider 'valid'
  // pressure of 0 means no pressing!

  TSPoint oriPos( p );

  if( p.z > MINPRESSURE && p.z < MAXPRESSURE ) {
    tft.fillScreen(BLACK);
    g_cali.x += ( oriPos.x - g_check[ g_checkIndex ][ 2 ] );
    g_cali.y += ( oriPos.y - g_check[ g_checkIndex ][ 3 ] );
    ++g_checkIndex;
    if( 5 <= g_checkIndex ) {
      g_cali.x /= g_checkIndex;
      g_cali.y /= g_checkIndex;
      tft.setCursor(20, 50);
      tft.setTextSize(2);
      tft.setTextColor( RED );
      tft.print( "X:" );
      tft.print( g_cali.x );
      tft.print( " Y:" );
      tft.print( g_cali.y );
      g_checkIndex = 0;
      g_cali.x = 0;
      g_cali.y = 0;
    }
    tft.fillCircle( g_check[ g_checkIndex ][0], g_check[ g_checkIndex ][1], PENRADIUS, g_colors[ g_checkIndex ] );
  }
}
