#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_TFTLCD.h> // Hardware-specific library
#include <TouchScreen.h>

#if defined(__SAM3X8E__)
    #undef __FlashStringHelper::F(string_literal)
    #define F(string_literal) string_literal
#endif

#define CALIBRATION_X 125
#define CALIBRATION_Y 80

// When using the BREAKOUT BOARD only, use these 8 data lines to the LCD:
// For the Arduino Uno, Duemilanove, Diecimila, etc.:
//   D0 connects to digital pin 8  (Notice these are
//   D1 connects to digital pin 9   NOT in order!)
//   D2 connects to digital pin 2
//   D3 connects to digital pin 3
//   D4 connects to digital pin 4
//   D5 connects to digital pin 5
//   D6 connects to digital pin 6
//   D7 connects to digital pin 7

// For the Arduino Mega, use digital pins 22 through 29
// (on the 2-row header at the end of the board).
//   D0 connects to digital pin 22
//   D1 connects to digital pin 23
//   D2 connects to digital pin 24
//   D3 connects to digital pin 25
//   D4 connects to digital pin 26
//   D5 connects to digital pin 27
//   D6 connects to digital pin 28
//   D7 connects to digital pin 29

// For the Arduino Due, use digital pins 33 through 40
// (on the 2-row header at the end of the board).
//   D0 connects to digital pin 33
//   D1 connects to digital pin 34
//   D2 connects to digital pin 35
//   D3 connects to digital pin 36
//   D4 connects to digital pin 37
//   D5 connects to digital pin 38
//   D6 connects to digital pin 39
//   D7 connects to digital pin 40
/*
#define YP 9  // must be an analog pin, use "An" notation!
#define XM 8  // must be an analog pin, use "An" notation!
#define YM A2   // can be a digital pin
#define XP A3   // can be a digital pin
*/

#define YP A3  // must be an analog pin, use "An" notation!
#define XM A2  // must be an analog pin, use "An" notation!
#define YM 9   // can be a digital pin
#define XP 8   // can be a digital pin

#define TS_MINX 0
#define TS_MINY 0
#define TS_MAXX 800
#define TS_MAXY 820

// For better pressure precision, we need to know the resistance
// between X+ and X- Use any multimeter to read it
// For the one we're using, its 300 ohms across the X plate
TouchScreen ts = TouchScreen( XP, YP, XM, YM, 300 );

#define LCD_CS A3
#define LCD_CD A2
#define LCD_WR A1
#define LCD_RD A0
// optional
#define LCD_RESET A4

// Assign human-readable names to some common 16-bit color values:
#define	BLACK   0x0000
#define	BLUE    0x001F
#define	RED     0xF800
#define	GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

Adafruit_TFTLCD tft( LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET );

// { pos, 0(top) 1(bottom) }
// ICE, FIRE, CLAY, HEAD, EYE
int g_cells[5][2] =
{
  { 0, 0 },
  { 1, 0 },
  { 2, 0 },
  { 3, 0 },
  { 4, 0 },
};

int g_crntPos( 0 );

#define BOXSIZE 64
#define HALFBOXSIZE 32

void setup(void) {
  Serial.begin( 9600 );

  tft.reset();
  
  uint16_t identifier( tft.readID() );
  if(identifier == 0x9325) {
    Serial.println(F("Found ILI9325 LCD driver"));
  } else if(identifier == 0x9328) {
    Serial.println(F("Found ILI9328 LCD driver"));
  } else if(identifier == 0x4535) {
    Serial.println(F("Found LGDP4535 LCD driver"));
  }else if(identifier == 0x7575) {
    Serial.println(F("Found HX8347G LCD driver"));
  } else if(identifier == 0x9341) {
    Serial.println(F("Found ILI9341 LCD driver"));
  } else if(identifier == 0x8357) {
    Serial.println(F("Found HX8357D LCD driver"));
  } else if(identifier==0x0101) {     
    identifier=0x9341;
    Serial.println(F("Found 0x9341 LCD driver"));
  } else {
    Serial.print(F("Unknown LCD driver chip: "));
    Serial.println(identifier, HEX);
    Serial.println(F("If using the Adafruit 2.8\" TFT Arduino shield, the line:"));
    Serial.println(F("  #define USE_ADAFRUIT_SHIELD_PINOUT"));
    Serial.println(F("should appear in the library header (Adafruit_TFT.h)."));
    Serial.println(F("If using the breakout board, it should NOT be #defined!"));
    Serial.println(F("Also if using the breakout, double-check that all wiring"));
    Serial.println(F("matches the tutorial."));
    identifier=0x9341;
  }

  tft.begin( identifier );
  tft.setRotation( 2 );
  _reset();
  pinMode( 13, OUTPUT );
}

#define MINPRESSURE 100
#define MAXPRESSURE 2000

TSPoint _calcPos( bool in_up, int in_pos )
{
  int y( in_pos * BOXSIZE );
  int x( 0 );
  if( in_up ) {
    x = 240 - BOXSIZE;
  }
  TSPoint pos( x, y, 0 );
  return pos;
}

void _ice( bool in_up, int in_pos )
{
  TSPoint pos( _calcPos( in_up, in_pos ) );
  tft.fillRect( pos.x, pos.y, BOXSIZE, BOXSIZE, BLACK );

  tft.fillCircle( pos.x + BOXSIZE / 2, pos.y + BOXSIZE / 2, 30, 0xefdf );
  tft.drawTriangle( pos.x + 8, pos.y + 15, pos.x + 56, pos.y + 15, pos.x + BOXSIZE / 2, pos.y + 60, 0x0ab4 );
  tft.fillTriangle( pos.x + 17, pos.y + 20, pos.x + 47, pos.y + 20, pos.x + BOXSIZE / 2, pos.y + 50, 0x0ab4 );
}

void _fire( bool in_up, int in_pos )
{
  TSPoint pos( _calcPos( in_up, in_pos ) );
  tft.fillRect( pos.x, pos.y, BOXSIZE, BOXSIZE, BLACK );

  tft.fillCircle( pos.x + BOXSIZE / 2, pos.y + BOXSIZE / 2, 30, 0xf774 );
  tft.drawRect( pos.x + 10, pos.y + 15, 44, 12, 0xc344 );
  tft.fillRect( pos.x + 12, pos.y + 17, 40, 8, 0xc344 );

  tft.drawRect( pos.x + 10, pos.y + 37, 44, 12, 0xc344 );
  tft.fillRect( pos.x + 12, pos.y + 39, 40, 8, 0xc344 );
}

void _clay( bool in_up, int in_pos )
{
  TSPoint pos( _calcPos( in_up, in_pos ) );
  tft.fillRect( pos.x, pos.y, BOXSIZE, BOXSIZE, BLACK );

  tft.fillTriangle( pos.x + BOXSIZE - 5, pos.y + HALFBOXSIZE - 5, pos.x + BOXSIZE - 25, pos.y + 10, pos.x + HALFBOXSIZE + 5, pos.y + HALFBOXSIZE, 0xf4a4 );
  tft.fillTriangle( pos.x + BOXSIZE - 25, pos.y + HALFBOXSIZE - 5, pos.x + BOXSIZE - 40, pos.y + 10, pos.x + HALFBOXSIZE - 20, pos.y + HALFBOXSIZE, 0xf4a4 );

  tft.fillTriangle( pos.x + BOXSIZE - 5, pos.y + HALFBOXSIZE + 5, pos.x + BOXSIZE - 25, pos.y + BOXSIZE - 10, pos.x + HALFBOXSIZE + 5, pos.y + HALFBOXSIZE, 0xf4a4 );
  tft.fillTriangle( pos.x + BOXSIZE - 25, pos.y + HALFBOXSIZE + 5, pos.x + BOXSIZE - 40, pos.y + BOXSIZE - 10, pos.x + HALFBOXSIZE - 20, pos.y + HALFBOXSIZE, 0xf4a4 );

  tft.fillTriangle( pos.x + HALFBOXSIZE + 10, pos.y + HALFBOXSIZE - 10, pos.x + HALFBOXSIZE + 10, pos.y + HALFBOXSIZE + 10, pos.x, pos.y + HALFBOXSIZE, 0xf4a4 );
}

void _head( bool in_up, int in_pos )
{
  TSPoint pos( _calcPos( in_up, in_pos ) );
  tft.fillRect( pos.x, pos.y, BOXSIZE, BOXSIZE, BLACK );
  
  tft.fillTriangle( pos.x + 15, pos.y + 5, pos.x + 49, pos.y + 5, pos.x + HALFBOXSIZE, pos.y + 20, YELLOW );
  tft.fillTriangle( pos.x + 5, pos.y + 15, pos.x + 5, pos.y + 49, pos.x + 20, pos.y + HALFBOXSIZE, YELLOW );
  tft.fillTriangle( pos.x + 59, pos.y + 15, pos.x + 59, pos.y + 49, pos.x + 44, pos.y + HALFBOXSIZE, YELLOW );
  tft.fillTriangle( pos.x + 15, pos.y + 59, pos.x + 49, pos.y + 59, pos.x + HALFBOXSIZE, pos.y + 44, YELLOW );
}

void _eye( bool in_up, int in_pos )
{
  TSPoint pos( _calcPos( in_up, in_pos ) );
  tft.fillRect( pos.x, pos.y, BOXSIZE, BOXSIZE, BLACK );

  tft.fillTriangle( pos.x + HALFBOXSIZE, pos.y, pos.x + 10, pos.y + HALFBOXSIZE, pos.x + BOXSIZE - 10, pos.y + HALFBOXSIZE, 0xab8e );
  tft.fillTriangle( pos.x + HALFBOXSIZE, pos.y + BOXSIZE, pos.x + 10, pos.y + HALFBOXSIZE, pos.x + BOXSIZE - 10, pos.y + HALFBOXSIZE, 0xab8e );
  tft.fillCircle( pos.x + HALFBOXSIZE, pos.y + HALFBOXSIZE, 16, RED );
  tft.fillCircle( pos.x + HALFBOXSIZE, pos.y + HALFBOXSIZE, 10, 0xf638 );
}

void _reset()
{
  tft.fillScreen( BLACK );
  for( int cnt( 0 ); cnt < 5; ++cnt ) {
    g_cells[ cnt ][ 0 ] = cnt;
    g_cells[ cnt ][ 1 ] = 0;
  }

  for( int cnt( 0 ); cnt < 5; ++cnt ) {
    _drawCard( cnt );
  }
  g_crntPos = 0;
}

void _erase( int in_kind )
{
    int pos( g_cells[ in_kind ][ 0 ] );
    bool isUp( 0 == g_cells[ in_kind ][ 1 ] );
    TSPoint boxPos( _calcPos( isUp, pos ) );
    tft.fillRect( boxPos.x, boxPos.y, BOXSIZE, BOXSIZE, BLACK );
}

void _checkCard( int in_x, int in_y )
{
  bool touchUp( false ); 

  for( int cnt( 0 ); cnt < 5; ++cnt ) {
    int pos( g_cells[ cnt ][ 0 ] );
    bool isUp( 0 == g_cells[ cnt ][ 1 ] );
    TSPoint boxPos( _calcPos( isUp, pos ) );
    if( boxPos.x > in_x || ( boxPos.x + BOXSIZE ) < in_x 
      || boxPos.y > in_y || ( boxPos.y + BOXSIZE ) < in_y 
    ) {
      continue;
    }
    _swap( cnt );
    if( isUp ) {
      touchUp = true;
    }
    break;
  }

  if( !touchUp ) {
    return;
  }

  int remainUp( -1 );
  int numRemainUp( 0 );

  for( int cnt( 0 ); cnt < 5; ++cnt ) {
    int pos( g_cells[ cnt ][ 0 ] );
    bool isUp( 0 == g_cells[ cnt ][ 1 ] );
    if( isUp ) {
      remainUp = cnt;
      ++numRemainUp;
    }
  }

  if( 1 == numRemainUp ) {
    _swap( remainUp );
  }
}

void _checkReset( int in_x, int in_y )
{
  int margin( 5 );
  if( BOXSIZE + margin < in_x && in_x < 240 - BOXSIZE - margin ) {
    _reset();
  }
}

void _swap( int in_kind )
{
  _erase( in_kind );
  int erasePos( g_cells[ in_kind ][ 0 ] );

  if( 0 == g_cells[ in_kind ][ 1 ] ) { // Up
    g_cells[ in_kind ][ 1 ] = 1;
    g_cells[ in_kind ][ 0 ] = g_crntPos;
    ++g_crntPos;
  } else { // Down
    bool reDraw( false );
    g_cells[ in_kind ][ 1 ] = 0; // to Up
    reDraw = g_cells[ in_kind ][ 0 ] != g_crntPos;
    g_cells[ in_kind ][ 0 ] = in_kind;
    if( reDraw ) {
      _reDraw( in_kind, erasePos );
    } else {
      --g_crntPos;
    }
  }
  _drawCard( in_kind );
}

void _drawCard( int in_kind )
{
    int pos( g_cells[ in_kind ][ 0 ] );
    bool isUp( 0 == g_cells[ in_kind ][ 1 ] );
    switch( in_kind ) {
    case 0:
      _ice( isUp, pos );
      break;  
    case 1:
      _fire( isUp, pos );
      break;  
    case 2:
      _clay( isUp, pos );
      break;  
    case 3:
      _head( isUp, pos );
      break;  
    case 4:
      _eye( isUp, pos );
      break;  
    }
}

void _reDraw( int in_kind, int in_erasePos )
{
  g_crntPos = 0;
  for( int cnt( 0 ); cnt < 5; ++cnt ) {
    if( 0 < g_cells[ cnt ][ 1 ] ) {
      if( in_erasePos < g_cells[ cnt ][ 0 ] ) {
        --g_cells[ cnt ][ 0 ];
        _drawCard( cnt );
      }
      ++g_crntPos;
    }
  }
  _eraseTail();
}

void _eraseTail()
{
    TSPoint boxPos( _calcPos( false, g_crntPos ) );
    int size( 5 - g_crntPos );
    tft.fillRect( boxPos.x, boxPos.y, BOXSIZE, BOXSIZE * size, BLACK );
}

void loop()
{
  digitalWrite( 13, HIGH );
  TSPoint p( ts.getPoint() );
  digitalWrite( 13, LOW );

  // if sharing pins, you'll need to fix the directions of the touchscreen pins
  //pinMode(XP, OUTPUT);
  pinMode( XM, OUTPUT );
  pinMode( YP, OUTPUT );
  //pinMode(YM, OUTPUT);

  // we have some minimum pressure we consider 'valid'
  // pressure of 0 means no pressing!
  p.x = p.x - CALIBRATION_X;
  if( p.x < 0 ) {
    p.x = 0;
  }
  p.y = p.y - CALIBRATION_Y;
  if( p.y < 0 ) {
    p.y = 0;
  }

  if( p.z > MINPRESSURE && p.z < MAXPRESSURE ) {
    // scale from 0->1023 to tft.width
    p.x = map(p.x, TS_MINX, TS_MAXX, tft.width(), 0);
    p.y = (tft.height()-map(p.y, TS_MINY, TS_MAXY, tft.height(), 0));
    _checkCard( p.x, p.y );
    _checkReset( p.x, p.y );
  }
}
